/*:
 * @plugindesc Final Overrides Plugin
 * @author kin-kun
 * 
 * @param Reload on shift F5
 * @type boolean
 * @on YES
 * @off NO
 * @desc Do you want to change reload to shift F5?
 * @default true
 * 
 * @param Disable scale key when not scalable
 * @type boolean
 * @on YES
 * @off NO
 * @desc Do you want to disable the scale key when
 * the display won't scale anyways?
 * @default true
 * 
 * @param Item Text Width
 * @type number
 * @min 50
 * @max 32000
 * @desc When calculating column for the item list
 * each column must be at least this size
 * @default 100
 *
 * @param Skill Text Width
 * @type number
 * @min 50
 * @max 32000
 * @desc When calculating column for the skill list
 * each column must be at least this size
 * @default 100
 * 
 * @help Plugin to override specific commands
 * This should be the last plugin loaded
 * 
 * Show all lines of a message quickly \>\>
 * 
 * Used Yanfly Message Core as reference
 */


/*jshint esversion: 6 */

var Imported = Imported || {}; // jshint ignore:line
Imported.FinalOverrides = "1.0";
var FinalOverrides = FinalOverrides || {}; // jshint ignore:line

(function ($) {
	"use strict";

	// kinkun.FinalOverrides._Window_Base_convertEscapeCharacters = Window_Base.prototype.convertEscapeCharacters;
	// Window_Base.prototype.convertEscapeCharacters = function (text) {

	// 	text = kinkun.FinalOverrides._Window_Base_convertEscapeCharacters.call(this, text);
	// 	text = this.convertExtra(text);
	// 	return text;
	// };

	// Window_Base.prototype.convertExtra = function (text) {
	// 	// Font Codes
	// 	text = text.replace(/\x1bFR/gi, '\x1bF[0]');
	// 	text = text.replace(/\x1bFB/gi, '\x1bF[1]');
	// 	text = text.replace(/\x1bFI/gi, '\x1bF[2]');

	// 	return text;
	// };


	// kinkun.FinalOverrides._Window_Base_processEscapeCharacter =
	// 	Window_Base.prototype.processEscapeCharacter;
	// Window_Base.prototype.processEscapeCharacter = function (code, textState) {
	// 	switch (code) {
	// 		case 'F':
	// 			let id = this.obtainEscapeParam(textState);
	// 			if (id === 0) {
	// 				$gameSystem.initMessageFontSettings();
	// 				this.resetFontSettings();
	// 			}
	// 			if (id === 1) this.contents.fontBold = !this.contents.fontBold;
	// 			if (id === 2) this.contents.fontItalic = !this.contents.fontItalic;
	// 			break;
	// 		case 'FS':
	// 			let size = this.obtainEscapeParam(textState);
	// 			this.contents.fontSize = size;
	// 			break;
	// 		case 'OC':
	// 			let color = this.obtainEscapeParam(textState);
	// 			this.contents.outlineColor = this.textColor(color);
	// 			break;
	// 		case 'OW':
	// 			this.contents.outlineWidth = this.obtainEscapeParam(textState);
	// 			break;
	// 		case 'PX':
	// 			textState.x = this.obtainEscapeParam(textState);
	// 			break;
	// 		case 'PY':
	// 			textState.y = this.obtainEscapeParam(textState);
	// 			break;
	// 		default:
	// 			kinkun.FinalOverrides._Window_Base_processEscapeCharacter.call(this,
	// 				code, textState);
	// 			break;
	// 	}
	// };
	let m_changeRefresh = true;
	let m_changeScale = true;
	let m_itemTextWidth = 100;
	let m_skillTextWidth = 100;

	function GetNumber(item, defaultValue)
	{
		let val = parseInt(item);
		if (isNaN(val))
		{
			return defaultValue;
		}
		return val;
	}

	function Init() {
		let params = PluginManager.parameters("Final Overrides");
		m_changeRefresh = String(params["Reload on shift F5"]).toLocaleLowerCase() === String(true).toLocaleLowerCase();
		m_changeScale = String(params["Disable scale key when not scalable"]).toLocaleLowerCase() === String(true).toLocaleLowerCase();
		m_itemTextWidth = GetNumber(String(params["Item Text Width"]), m_itemTextWidth);
		m_skillTextWidth = GetNumber(String(params["Skill Text Width"]), m_skillTextWidth);
		AddEventListeners();
	}

	Window_ShopNumber.prototype.createButtons = function () {
		let bitmap = ImageManager.loadSystem('ButtonSet');
		let buttonWidth = bitmap.width / 6;
		let buttonHeight = bitmap.height / 2;
		this._buttons = [];
		for (let i = 0; i < 5; i++) {
			let button = new Sprite_Button();
			let x = buttonWidth * i;
			let w = buttonWidth * (i === 4 ? 2 : 1);
			button.bitmap = bitmap;
			button.setColdFrame(x, 0, w, buttonHeight);
			button.setHotFrame(x, buttonHeight, w, buttonHeight);
			button.visible = false;
			this._buttons.push(button);
			this.addChild(button);
		}
		this._buttons[0].setClickHandler(this.onButtonDown2.bind(this));
		this._buttons[1].setClickHandler(this.onButtonDown.bind(this));
		this._buttons[2].setClickHandler(this.onButtonUp.bind(this));
		this._buttons[3].setClickHandler(this.onButtonUp2.bind(this));
		this._buttons[4].setClickHandler(this.onButtonOk.bind(this));
	};

	Window_ShopNumber.prototype.placeButtons = function () {
		let numButtons = this._buttons.length;
		let spacing = 16;
		let totalWidth = -spacing;
		for (let i = 0; i < numButtons; i++) {
			totalWidth += this._buttons[i].width + spacing;
		}
		let x = (this.width - totalWidth) / 2;
		for (let j = 0; j < numButtons; j++) {
			let button = this._buttons[j];
			button.x = x;
			button.y = this.buttonY() - spacing;
			x += button.width + spacing;
		}
	};

	function CalculateColumns(width, itemWidth)
	{
		let colcount = Math.floor(width / itemWidth);
		if (colcount <= 0)
			colcount = 1;
		return colcount;
	}

	Window_ItemList.prototype.maxCols = function () {

		return CalculateColumns(this.width, m_itemTextWidth);
	};

	Window_SkillList.prototype.maxCols = function () {
		return CalculateColumns(this.width, m_skillTextWidth);
	};

	$._updatePlacement = Window_EventItem.prototype.updatePlacement;
	Window_EventItem.prototype.updatePlacement = function () {
		if (this._messageWindow.y >= Graphics.boxHeight / 2) {
			$._updatePlacement.call(this);
		} else {
			this.height = 120;
			this.y = Graphics.boxHeight - this.height - 75;
			this.x = 50;
			this.width = 500;
		}
	};

	$._processEscapeCharacter = Window_Message.prototype.processEscapeCharacter;
	Window_Message.prototype.processEscapeCharacter = function (code, textState) {
		switch (code) {
			case 'T':
				this._lineShowFast = true;
				textState.x += 160;
				this.height = 130;
				this.y = Graphics.boxHeight - this.height;
				break;
			case '>':
				if (this._lineShowFast)
					this._lineShowFastAll = true;
				break;
			case '<':
				if (this._lineShowFast)
					this._lineShowFastAll = false;
				break;
		}
		$._processEscapeCharacter.call(this, code, textState);
	};

	$._processNewLine = Window_Message.prototype.processNewLine;
	Window_Message.prototype.processNewLine = function (textState) {
		$._processNewLine.call(this, textState);
		this._lineShowFast = this._lineShowFastAll || false;
	};

	$._startMessage = Window_Message.prototype.startMessage;
	Window_Message.prototype.startMessage = function (textState) {
		this._lineShowFastAll = false;
		$._startMessage.call(this, textState);
	};

	/*
	 * Look specifically for \n (lowercase) and return the
	 * code N so it is treated as a newline.
		   */
	$._obtainEscapeCode = Window_Base.prototype.obtainEscapeCode;
	Window_Base.prototype.obtainEscapeCode = function (textState) {
		let regExp = /^n/;
		let arr = regExp.exec(textState.text.slice(textState.index + 1));
		if (arr) {
			textState.index += arr[0].length + 1;
			return "N";
		} else {
			return $._obtainEscapeCode(textState);
		}
	};

	Scene_Map.prototype.isDebugCalled = function () {
		return Input.isTriggered('debug');
	};

	// Override the bgm fadeout for main effects
	// used by summer memories
	AudioManager.playMe = function (me) {
		this.stopMe();
		if (me.name) {
			this._meBuffer = this.createBuffer('me', me.name);
			this.updateMeParameters(me);
			this._meBuffer.play(false);
			this._meBuffer.addStopListener(this.stopMe.bind(this));
		}
	};

	function AddEventListeners() {
		document.addEventListener("keydown", (keyboardEvent) => {
			if (keyboardEvent.ctrlKey)
				return;
			if (keyboardEvent.altKey)
				return;
			if (keyboardEvent.metaKey)
				return;
			if (m_changeRefresh) {
				if (keyboardEvent.shiftKey) {
					if (keyboardEvent.key === "F5" && m_changeRefresh)
						document.location.reload();
					return;
				}
				if (keyboardEvent.key === "F5") {
					keyboardEvent.stopImmediatePropagation();
					return;
				}
			}

			if (m_changeScale) {
				// Disable F3 when not stretched (as it appears to do nothing until you resize which kinda sucks)
				if (keyboardEvent.key === "F3") {
					if (window.innerWidth === Graphics.boxWidth && window.innerHeight === Graphics.boxHeight) {
						Graphics._stretchEnabled = Graphics._defaultStretchMode();
						keyboardEvent.stopImmediatePropagation();
					}
				}
			}
		});
	}

	Init();

})(FinalOverrides);