/*:
 * @plugindesc Load and Save Cache Plugin
 * @author kin-kun
 * 
 * @help Plugin to cache global data when loading the save/load screen, this
 * prevents in some cases nearly 1 million filesystem calls
 * 
 * Changelog:
 * v1.1
 * Fixed calls to reserve images when not needed
 * Fixed inefficient code for finding the last modified save
 */


/*jshint esversion: 6 */

var Imported = Imported || {}; // jshint ignore:line
if (!Imported.LoadAndSaveCache) {
	Imported.LoadAndSaveCache = "1.1";
	var LoadAndSaveCache = LoadAndSaveCache || {}; // jshint ignore:line

	(function ($) {
		"use strict";

		$._globalInfo = undefined;
		$.DataManager_loadGlobalInfo = DataManager.loadGlobalInfo;
		DataManager.loadGlobalInfo = function () {
			if (!$._globalInfo) {
				$._globalInfo = $.DataManager_loadGlobalInfo.apply(this, arguments);
			}
			return $._globalInfo;
		};

		DataManager.latestSavefileId = function () {
			const globalInfo = this.loadGlobalInfo();
			if (globalInfo) {
				const mapped = globalInfo.map((v, i) => {
					return { i, value: v && v.timestamp || 0 };
				}).sort((a, b) => (a.value < b.value) ? 1 : ((b.value < a.value) ? -1 : 0));
				for (const el of mapped) {
					if (el.i !== 0 && this.isThisGameFile(el.i)) {
						console.timeEnd("DataManager.latestSavefileId");
						return el.i;
					}
				}
			}
			return 1;
		};

		DataManager.loadSavefileImages = function (info) {
			return;
		};
		
	})(LoadAndSaveCache);
}