/*:
 * @plugindesc Touches Mouse Tracking
 * @author kin-kun
 * 
 * @help Update the mouse coordinates on touches for games like Summer Memories
 */


/*jshint esversion: 6 */

var Imported = Imported || {}; // jshint ignore:line
if (!Imported.TouchesMouseTracking) {
	Imported.TouchesMouseTracking = "1.0";
	var TouchesMouseTracking = TouchesMouseTracking || {}; // jshint ignore:line

	(function ($) {
		"use strict";

        $._TouchInput__onTouchStart = TouchInput._onTouchStart;
        $._TouchInput__onTouchMove = TouchInput._onTouchMove;
        $._TouchInput__onTouchEnd = TouchInput._onTouchEnd;

        TouchInput._TouchEvent = function (event, action){
            let firstTouch;
            if (event) {
                if (event.changedTouches) {
                    for (let i = 0; i < event.changedTouches.length; i++) {
                        const touch = event.changedTouches[i];
                        if (!touch) continue;
                        const x = Graphics.pageToCanvasX(touch.pageX);
                        const y = Graphics.pageToCanvasY(touch.pageY);
                        if (Graphics.isInsideCanvas(x, y)) {
                            this.mouseX = x;
                            this.mouseY = y;
                            action.call(this, event);
                            return;
                        }
                    }
                }
                if (event.touches) {
                    for (let i = 0; i < event.touches.length; i++) {
                        const touch = event.touches[i];
                        if (!touch) continue;
                        if (!firstTouch) { firstTouch = touch; }
                        const x = Graphics.pageToCanvasX(touch.pageX);
                        const y = Graphics.pageToCanvasY(touch.pageY);
                        if (Graphics.isInsideCanvas(x, y)) {
                            this.mouseX = x;
                            this.mouseY = y;
                            action.call(this, event);
                            return;
                        }
                    }
                }
                if (firstTouch) {
                    this.mouseX = Graphics.pageToCanvasX(firstTouch.pageX);
                    this.mouseY = Graphics.pageToCanvasY(firstTouch.pageY);
                }
            }
            action.call(this, event);
        };

        TouchInput._onTouchStart = function (event) {
            this._TouchEvent(event, $._TouchInput__onTouchStart);
        };

        TouchInput._onTouchMove = function (event) {
            this._TouchEvent(event, $._TouchInput__onTouchMove);
        };

        TouchInput._onTouchEnd = function (event) {
            this._TouchEvent(event, $._TouchInput__onTouchEnd);
        };



	})(TouchesMouseTracking);
}